//
//  FlipsideViewController.m
//  FlippingPreferableClock
//
//  Created by Chris Adamson on 10/30/08.
//  Copyright Subsequently and Furthermore, Inc. 2008. All rights reserved.
//
//
//  Licensed with the Apache 2.0 License
//  http://apache.org/licenses/LICENSE-2.0
//


#import "FlipsideViewController.h"


@implementation FlipsideViewController


//START:code.FlippingPreferableClock.loadtimezonenames
-(void) loadTimeZoneNames {
	NSArray *unsortedTimeZoneNames = [NSTimeZone knownTimeZoneNames];
	timeZoneNames = 
		[unsortedTimeZoneNames sortedArrayUsingSelector:
		 @selector(caseInsensitiveCompare:)];
	[unsortedTimeZoneNames release];
	[timeZoneNames retain];
}
//END:code.FlippingPreferableClock.loadtimezonenames


- (void)viewDidLoad {
    [super viewDidLoad];
    self.view.backgroundColor = [UIColor viewFlipsideBackgroundColor];  
    
	[self loadTimeZoneNames];
}


- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}


- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning]; // Releases the view if it doesn't have a superview
    // Release anything that's not essential, such as cached data
}


- (void)dealloc {
    [super dealloc];
}


-(NSString*) selectedTimeZone {
	return [timeZoneNames objectAtIndex: [timeZonePicker selectedRowInComponent: 0]];
}

-(void) setSelectedTimeZone: (NSString* )tz {
	NSUInteger tzIndex = [timeZoneNames indexOfObject: tz];
	if (tzIndex != NSNotFound)
		[timeZonePicker selectRow: tzIndex inComponent: 0 animated: NO];
}

-(BOOL) uses24Hour {
	return twentyFourHourSwitch.on;
}

-(void) setUses24Hour: (BOOL) u24h {
	twentyFourHourSwitch.on = u24h;
}


//START:code.FlippingPreferableClock.pickerimplementation
-(NSInteger) pickerView: (UIPickerView*) pickerView
			numberOfRowsInComponent: (NSInteger) component {
	return [timeZoneNames count];
}

- (NSInteger) numberOfComponentsInPickerView:(UIPickerView *)pickerView {
	return 1;
}

- (NSString *)pickerView:(UIPickerView *)pickerView
			titleForRow:(NSInteger)row
			forComponent:(NSInteger)component {
	return (NSString*) [timeZoneNames objectAtIndex: row];
}
//END:code.FlippingPreferableClock.pickerimplementation

@end
