//
//  MainViewController.m
//  FlippingPreferableClock
//
//  Created by Chris Adamson on 10/30/08.
//  Copyright Subsequently and Furthermore, Inc. 2008. All rights reserved.
//
//
//  Licensed with the Apache 2.0 License
//  http://apache.org/licenses/LICENSE-2.0
//


#import "MainViewController.h"
#import "MainView.h"

@implementation MainViewController

NSString* timeZoneName = NULL;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
        // Custom initialization
    }
    return self;
}



//START:code.FlippingPreferableClock.udpateclockview
-(void) updateClockView {
	if (clockFormatter == NULL) {
		timeLabel.text = @"";
		timeZoneLabel.text = @"";
		return;
	}
	NSDate *dateNow = [NSDate date];
	timeLabel.text = [clockFormatter stringFromDate: dateNow];
	timeZoneLabel.text = timeZoneName;
}
//END:code.FlippingPreferableClock.udpateclockview

//START:code.FlippingPreferableClock.setupnstimer
-(void) startClock {
	// since timer's first callback will occur after interval,
	// do one up-front refresh
	[self updateClockView];
	// now set up timer to repeatedly call updateClockView
	clockViewUpdateTimer = [NSTimer scheduledTimerWithTimeInterval:0.2
				target:self
				selector:@selector (updateClockView)
				userInfo:NULL
				repeats:YES];
}
//END:code.FlippingPreferableClock.setupnstimer


/*
// Implement viewDidLoad to do additional setup after loading the view.
 */
- (void)viewDidLoad {
    [super viewDidLoad];
	[self startClock];
}


- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}


- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning]; // Releases the view if it doesn't have a superview
    // Release anything that's not essential, such as cached data
}


- (void)dealloc {
    [super dealloc];
}

-(void) setClockToTimeZoneName: (NSString*) tz uses24Hour: (BOOL) u24h {
	if (timeZoneName != NULL)
		[timeZoneName release];
	[tz retain];
	timeZoneName = tz;
	// set time formatter with 24 hour preference and time zone
	if (clockFormatter == nil) {
		clockFormatter = [[NSDateFormatter alloc] init];
	}
	// see formats at http://unicode.org/reports/tr35/tr35-4.html#Date_Format_Patterns
	[clockFormatter setTimeZone: [NSTimeZone timeZoneWithName: tz]];
	if (u24h)
		[clockFormatter setDateFormat: @"H:mm:ss"];
	else
		[clockFormatter setDateFormat: @"h:mm:ss a"];
}
@end
