//
//  RootViewController.m
//  FlippingPreferableClock
//
//  Created by Chris Adamson on 10/30/08.
//  Copyright Subsequently and Furthermore, Inc. 2008. All rights reserved.
//

#import "RootViewController.h"
#import "MainViewController.h"
#import "FlipsideViewController.h"


@implementation RootViewController

@synthesize infoButton;
@synthesize flipsideNavigationBar;
@synthesize mainViewController;
@synthesize flipsideViewController;

//START:code.FlippingPreferableClock.keysandconsts
NSString *TwentyFourHourPrefKey = @"24HourDisplay";
NSString *TimeZonePrefKey = @"TimeZone";
NSString *DefaultTwentyFourHourPref = @"NO";
NSString *DefaultTimeZonePref = @"America/Detroit";
NSMutableDictionary *clockPrefs;
NSString *prefsFilePath;
//END:code.FlippingPreferableClock.keysandconsts





// our new stuff is below

-(void) setMainViewFromPrefs {
	NSString* prefTimeZone = [clockPrefs objectForKey: TimeZonePrefKey];
	BOOL uses24Hour = [(NSString*)
			[clockPrefs objectForKey: TwentyFourHourPrefKey] boolValue];
	[self.mainViewController setClockToTimeZoneName:
			prefTimeZone uses24Hour: uses24Hour];
}

-(void) setFlipsideViewFromPrefs {
	NSString* prefTimeZone = [clockPrefs objectForKey: TimeZonePrefKey];
	BOOL uses24Hour = [(NSString*)
					   [clockPrefs objectForKey: TwentyFourHourPrefKey] boolValue];
	[flipsideViewController setSelectedTimeZone: prefTimeZone];
	[flipsideViewController setUses24Hour: uses24Hour];
}

/* This is a more appropriate means of finding the Documents directory, but
	we don't introduce NSSearchPathForDirectoriesInDomains until the db chapter.
- (void) initPrefsFilePath {
	NSArray *paths = NSSearchPathForDirectoriesInDomains
	(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDirectory = [paths objectAtIndex: 0];
	prefsFilePath = [documentsDirectory stringByAppendingPathComponent:
					 @"flippingprefs.plist"];
	[prefsFilePath retain];
}
 
 */

//START:code.FlippingPreferableClock.initandloadprefs
- (void) initPrefsFilePath {
	NSString *documentsDirectory =
		[NSHomeDirectory() stringByAppendingPathComponent:@"Documents"];
	prefsFilePath = [documentsDirectory stringByAppendingPathComponent:
					 @"flippingprefs.plist"];
	[prefsFilePath retain];
}

- (void) loadPrefs {
	if (prefsFilePath == nil)
		[self initPrefsFilePath];
	if ([[NSFileManager defaultManager] fileExistsAtPath: prefsFilePath]) {
		clockPrefs = [[NSMutableDictionary alloc]
					  initWithContentsOfFile: prefsFilePath];
		NSLog (@"read prefs from %@", prefsFilePath);
	}
	else {
		NSLog (@"creating default prefs");
		clockPrefs = [[NSMutableDictionary alloc] initWithCapacity: 2];
		[clockPrefs setObject: DefaultTimeZonePref
					   forKey: TimeZonePrefKey];
		[clockPrefs setObject: DefaultTwentyFourHourPref
					   forKey: TwentyFourHourPrefKey];
	}
}
//END:code.FlippingPreferableClock.initandloadprefs


//START:code.FlippingPreferableClock.saveprefs
-(void) savePrefs {
	// use widget states to set values in the prefs dictionary
	NSString *selectedTimeZone = [flipsideViewController selectedTimeZone];
	NSString *selected24HourDisplayS =
		[flipsideViewController uses24Hour] ?  @"YES" : @"NO";
	[clockPrefs setObject: selectedTimeZone forKey: TimeZonePrefKey];
	[clockPrefs setObject: selected24HourDisplayS
		forKey: TwentyFourHourPrefKey];

	// save prefs to documents folder
	[clockPrefs writeToFile: prefsFilePath atomically: YES];
}
//END:code.FlippingPreferableClock.saveprefs


// end our stuff




- (void)viewDidLoad {
    
    [super viewDidLoad];
    MainViewController *viewController = [[MainViewController alloc] initWithNibName:@"MainView" bundle:nil];
    self.mainViewController = viewController;
    [viewController release];

    // start our stuff
	
	// load user preferences and set views corresponsdingly
	[self loadPrefs];
	[self setMainViewFromPrefs];
	
	// end our stuff
	 
    [self.view insertSubview:mainViewController.view belowSubview:infoButton];
}


- (void)loadFlipsideViewController {
    
    FlipsideViewController *viewController = [[FlipsideViewController alloc] initWithNibName:@"FlipsideView" bundle:nil];
    self.flipsideViewController = viewController;
    [viewController release];
    
    // Set up the navigation bar
    UINavigationBar *aNavigationBar = [[UINavigationBar alloc] initWithFrame:CGRectMake(0.0, 0.0, 320.0, 44.0)];
    aNavigationBar.barStyle = UIBarStyleBlackOpaque;
    self.flipsideNavigationBar = aNavigationBar;
    [aNavigationBar release];
    
    UIBarButtonItem *buttonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:@selector(toggleView)];
    UINavigationItem *navigationItem = [[UINavigationItem alloc] initWithTitle:@"FlippingPreferableClock"];
    navigationItem.rightBarButtonItem = buttonItem;
    [flipsideNavigationBar pushNavigationItem:navigationItem animated:NO];
    [navigationItem release];
    [buttonItem release];
}


- (IBAction)toggleView {    
    /*
     This method is called when the info or Done button is pressed.
     It flips the displayed view from the main view to the flipside view and vice-versa.
     */
//START:code.FlippingPreferableClock.toggleviewsetup
    if (flipsideViewController == nil) {
        [self loadFlipsideViewController];
    }
    
    UIView *mainView = mainViewController.view;
    UIView *flipsideView = flipsideViewController.view;
    
    [UIView beginAnimations:nil context:NULL];
    [UIView setAnimationDuration:1];
    [UIView setAnimationTransition:([mainView superview] ?
			UIViewAnimationTransitionFlipFromRight :
				UIViewAnimationTransitionFlipFromLeft)
			forView:self.view cache:YES];
//END:code.FlippingPreferableClock.toggleviewsetup

    if ([mainView superview] != nil) {
        [flipsideViewController viewWillAppear:YES];
        [mainViewController viewWillDisappear:YES];
        [mainView removeFromSuperview];
        [infoButton removeFromSuperview];
        [self.view addSubview:flipsideView];
        [self.view insertSubview:flipsideNavigationBar aboveSubview:flipsideView];
        [mainViewController viewDidDisappear:YES];
        [flipsideViewController viewDidAppear:YES];
		[self setFlipsideViewFromPrefs]; // NEW: init flipside widgets
    } else {
		[self savePrefs];  // NEW: save prefs to disk
		[self setMainViewFromPrefs]; // NEW: update clock's tz and 12/24 behavior
        [mainViewController viewWillAppear:YES];
        [flipsideViewController viewWillDisappear:YES];
        [flipsideView removeFromSuperview];
        [flipsideNavigationBar removeFromSuperview];
        [self.view addSubview:mainView];
        [self.view insertSubview:infoButton aboveSubview:mainViewController.view];
        [flipsideViewController viewDidDisappear:YES];
        [mainViewController viewDidAppear:YES];
    }
    [UIView commitAnimations];
}



- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}


- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning]; // Releases the view if it doesn't have a superview
    // Release anything that's not essential, such as cached data
}


- (void)dealloc {
    [infoButton release];
    [flipsideNavigationBar release];
    [mainViewController release];
    [flipsideViewController release];
    [super dealloc];
}



@end
